/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: jonas-team@objectweb.org
 * Copyright (C) 2006 Distributed Systems Lab.
 * Universidad Politécnica de Madrid (Spain)
 * Contact: http://lsd.ls.fi.upm.es/lsd
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: HaServiceImpl.java 9297 2006-08-01 15:32:14Z frperezs $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jonas.ha;

import javax.management.InstanceNotFoundException;
import javax.management.MBeanRegistrationException;
import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.naming.Context;
import javax.naming.NamingException;

import org.objectweb.carol.cmi.configuration.TraceCmi;
import org.objectweb.carol.cmi.ha.ReplicationManager;
import org.objectweb.carol.cmi.ha.ReplicationManagerImpl;
import org.objectweb.carol.cmi.ha.interceptor.HACurrent;
import org.objectweb.carol.cmi.ha.interceptor.HACurrentDelegate;

import org.objectweb.jonas.ha.interceptor.HACurrentDelegateImpl;
import org.objectweb.jonas.jmx.J2eeObjectName;
import org.objectweb.jonas.jmx.JmxService;
import org.objectweb.jonas.jmx.JonasObjectName;
import org.objectweb.jonas.service.AbsServiceImpl;
import org.objectweb.jonas.service.ServiceException;
import org.objectweb.jonas.service.ServiceManager;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * This class binds the HA service to Jonas and initializes the HA architecture
 * @author Francisco Perez-Sorrosal (fpsorrosal@no-spam@fi.upm.es)
 * @author Alberto Paz-Jimenez (apaz@no-spam@fi.upm.es)
 * @author benoit pelletier
 */
public class HaServiceImpl extends AbsServiceImpl implements HaService, HaServiceImplMBean {

    private final String SERVICE_NAME = "HAService";
    
    private boolean started = false;

    private ReplicationManager replicationMgr;

    private HACurrentDelegate haCurrentDlg = null;

    private MBeanServer mbeanServer = null;
    
    public void doInit(Context ctx) throws ServiceException {
        TraceCmi.infoCmiHA("Initializing replication service...");
        try {
            
			replicationMgr = new ReplicationManagerImpl(ctx);
			
            // Init the HA current class
            haCurrentDlg = HACurrentDelegateImpl.getCurrent();
            HACurrent.getHACurrent().setDlg(haCurrentDlg);

            if (TraceCmi.isDebugCmiHA()) {
                TraceCmi.debugCmiHA("replicationMgr=" + replicationMgr);
                TraceCmi.debugCmiHA("haCurrentDlg=" + haCurrentDlg);
            }

            // Get the JMX Server via JMX Service
            try {
                mbeanServer =
                    ((JmxService) ServiceManager.getInstance().getJmxService()).getJmxServer();
            } catch (Exception e) {
            	TraceCmi.error("JOnAS: Cannot get JMX Service", e);
                mbeanServer = null;                
            }
        } catch (Exception e) {
            throw new ServiceException("Error in GCL name"); 
        }
    }

    public void doStart() throws ServiceException {
        if (mbeanServer != null) {
            try {
            	ObjectName on = JonasObjectName.haService();
            	mbeanServer.registerMBean(this, on);
            } catch (Exception e) {
            	TraceCmi.error("JOnAS: Cannot register HA mBean", e);
            	
            }            	
        }
        // COMPLETE: Allow to start and shutdown service dinamically
        started = true;
        TraceCmi.infoCmiHA("Replication service started");
    }

    public void doStop() throws ServiceException {
        // COMPLETE: Allow to start and shutdown service dinamically
        if (started) {
            if (mbeanServer != null) {
                // unregister HaMBean
                try {
                	ObjectName on = JonasObjectName.haService();
					mbeanServer.unregisterMBean(on);
				} catch (Exception e) {
					TraceCmi.error("JOnAS: Cannot unregister HA mBean", e);					
				}
            }
            started = false;
            replicationMgr.clear(); // Turn off the GCL
            TraceCmi.infoCmiHA("Replication service stopped");
        }
    }

    public boolean isStarted() {
        return started;
    }

    public void setName(String name) {
        // Nothing to do...
    }

    public String getName() {
        return SERVICE_NAME;
    }

    /**
     * Return the replication manager
     */
    public ReplicationManager getReplicationManager() {

        if (TraceCmi.isDebugCmiHA()) {
            TraceCmi.debugCmiHA("replicationMgr=" + replicationMgr);
        }

        return replicationMgr;
    }

    /**
     * MBean method:
     * Get the number of replicated messages sent. 
     */
	public long getNumberofReplicatedMessages() {

		return 0;
	}

    /**
     * MBean method:
     * Get the average size of the replicated messages sent.
     */
	public double getAvgSizeofReplicatedMessages() {

		return 0;
	}

    /**
     * MBean method:
	 * Get the SFSB info timeout to clean certain information stored in memory.
     */
	public long getSFSBInfoTimeout() {

		return 0;
	}

    /**
     * MBean method:
	 * Set the SFSB info timeout. The info stored in the node is removed when
	 * the timer expires.
	 * @param timeout the timeout to clean the info
     */
	public void setSFSBInfoTimeout(long timeout) {
		
	}

    /**
     * MBean method:
	 * Get the Datasource name required to hold the EB version numbers of the
	 * horizontal replication approach. 
     */
	public String getHRDatasourceName() {

		return "FooDS";
	}

    /**
     * MBean method:
	 * Set the Datasource name required to hold the EB version numbers of the
	 * horizontal replication approach.
	 * @param name the name of the datasource 
     */
	public void setHRDatasourceName(String name) {

		
	}
}
