/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2006 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: CmiClusterFactory.java 9402 2006-08-08 15:06:13Z durieuxp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.management.cluster;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;

import javax.management.JMException;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

import org.objectweb.jonas.management.monitoring.DomainMonitor;
import org.objectweb.jonas.management.monitoring.ServerProxy;

import org.objectweb.util.monolog.api.BasicLevel;

/**
 * Factory for CMI clusters
 * These Clusters are built dynamically, when a new server is discovered
 * as being part of a cluster of this type.
 * @author durieuxp
 */
public class CmiClusterFactory extends ClusterFactory {

    /**
     * List of TomcatCluster objects
     * There may be more than 1 TomcatCluster in the domain.
     * key = clusterName, value = TomcatCluster
     */
    private HashMap myclusters = new HashMap();

    /**
     * Constructor
     */
    public CmiClusterFactory(DomainMonitor dm) {
        super(dm);
    }

    /**
     * Look for a cluster by its name
     * @param name fo the cluster
     * @return cluster or null if not found
     */
    public BaseCluster getCluster(String name) {
        return (BaseCluster) myclusters.get(name);
    }

    /**
     * A new server has been discovered.
     * In case this server is recognized, it is added in a Cluster.
     * If not, nothing is done.
     * @param proxy The new ServerProxy
     * @return True if recognized as a tomcat server.
     */
    public boolean notifyServer(ServerProxy proxy) {

        String serverName = proxy.getServerName();
        logger.log(BasicLevel.DEBUG, serverName);

        // Determine if a CMI Cluster MBean exists in the server
        // referenced by this proxy.
        ObjectName ons;
        try {
            ons = ObjectName.getInstance(domainName + ":type=CMI,*");
        } catch (MalformedObjectNameException e1) {
            logger.log(BasicLevel.ERROR, "MalformedObjectNameException");
            return false;
        }
        Set set = proxy.queryNames(ons);
        if (set == null) {
            logger.log(BasicLevel.DEBUG, "Cannot reach " + serverName);
            return false;
        }
        if (set.isEmpty()) {
            logger.log(BasicLevel.DEBUG, "No CMI Cluster declared");
            return false;
        }
        // We must search for a MBean with name=JGroups
        boolean found = false;
        ObjectName on = null;
        for (Iterator it = set.iterator(); it.hasNext(); ) {
            on = (ObjectName) it.next();
            String name = on.getKeyProperty("name");
            if ("JGroups".equals(name)) {
                found = true;
                break;
            }
        }
        if (!found) {
            logger.log(BasicLevel.DEBUG, "No CMI Cluster with name=JGroups declared");
            return false;
        }

        // Found an MBean: Get information about this cluster
        // ObjectName = "<domain>:type=CMI,J2EEServer=<server>,name=JGroups,channel=<ch>,protocol=UDP"
        logger.log(BasicLevel.DEBUG, "Found CMICluster");
        String clusterName = on.getKeyProperty("channel");
        // TODO : get additional info : mcast addr, port, ...

        // Retrieve the CmiCluster. Create it if necessary.
        CmiCluster cluster = (CmiCluster) myclusters.get(clusterName);
        if (cluster == null) {
            ObjectName clon = null;
            try {
                cluster = new CmiCluster(this);
                clon = cluster.setName(clusterName);
            } catch (JMException e) {
                logger.log(BasicLevel.ERROR, "Cannot create CMI Cluster:" + e);
                return false;
            }

            // TODO: Set additional info

            // Register the MBean if not done
            if (!mbeanServer.isRegistered(clon)) {
                try {
                    // A MBean is registered for each Cluster
                    mbeanServer.registerMBean(cluster, clon);
                } catch (Exception e) {
                    logger.log(BasicLevel.ERROR, "Cannot register cluster:" + e);
                    return false;
                }
            }
            myclusters.put(clusterName, cluster);
        }

        // add a server to the cluster
        return cluster.addCmiServer(serverName, proxy);
    }

    public Collection getClusterList() {
        return myclusters.values();
    }

}
