/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2006 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: TomcatClusterFactory.java 9402 2006-08-08 15:06:13Z durieuxp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.management.cluster;

import java.util.Collection;
import java.util.HashMap;
import java.util.Set;

import javax.management.JMException;
import javax.management.ObjectName;

import org.objectweb.jonas.jmx.CatalinaObjectName;
import org.objectweb.jonas.management.monitoring.DomainMonitor;
import org.objectweb.jonas.management.monitoring.ServerProxy;

import org.objectweb.util.monolog.api.BasicLevel;

/**
 * Factory for clusters used by Tomact for Session Replication
 * These Clusters are built dynamically, when a new server is discovered
 * as being part of a cluster of this type.
 * @author durieuxp
 */
public class TomcatClusterFactory extends ClusterFactory {

    /**
     * List of TomcatCluster objects
     * There may be more than 1 TomcatCluster in the domain.
     * key = clusterName, value = TomcatCluster
     */
    private HashMap myclusters = new HashMap();

    /**
     * Constructor
     */
    public TomcatClusterFactory(DomainMonitor dm) {
        super(dm);
    }

    /**
     * Look for a cluster by its name
     * @param name fo the cluster
     * @return cluster or null if not found
     */
    public BaseCluster getCluster(String name) {
        return (BaseCluster) myclusters.get(name);
    }

    /**
     * A new server has been discovered.
     * In case this server is recognized, it is added in a Cluster.
     * If not, nothing is done.
     * @param proxy The new ServerProxy
     * @return True if recognized as a tomcat server.
     */
    public boolean notifyServer(ServerProxy proxy) {


        String serverName = proxy.getServerName();
        logger.log(BasicLevel.DEBUG, serverName);

        // Determine if a Tomcat Cluster MBean exists in the server
        // referenced by this proxy.
        // It should have been created by Tomcat in each server where the
        // tomcat HTTP replication has been defined (in server.xml)
        ObjectName clusterOns = CatalinaObjectName.catalinaClusters(domainName);
        Set clusterOnSet = proxy.queryNames(clusterOns);
        if (clusterOnSet == null) {
            logger.log(BasicLevel.DEBUG, "Cannot reach " + serverName);
            return false;
        }
        if (clusterOnSet.isEmpty()) {
            logger.log(BasicLevel.DEBUG, "No Tomcat Cluster declared");
            return false;
        }

        // A Server can be part of only 1 TomcatCluster : Take the first and only one.
        ObjectName clusterOn = (ObjectName) clusterOnSet.iterator().next();
        // CatalinaObjectName = "<domain>:type=Cluster,host=<hostname>"
        String hostName = clusterOn.getKeyProperty("host");
        String clusterName = (String) proxy.getAttribute(clusterOn, "clusterName");
        logger.log(BasicLevel.DEBUG, "Found TomcatCluster: " + hostName + "," + clusterName);

        // Retrieve the TomcatCluster. Create it if necessary.
        TomcatCluster cluster = (TomcatCluster) myclusters.get(clusterName);
        if (cluster == null) {
            ObjectName clon = null;
            try {
                cluster = new TomcatCluster(this);
                clon = cluster.setName(clusterName);
            } catch (JMException e) {
                logger.log(BasicLevel.ERROR, "Cannot create tomcat SessionCluster:" + e);
                return false;
            }

            // Get info about the catalina Cluster Membership.
            // It should be registered on the related server.
            ObjectName cmon = CatalinaObjectName.catalinaClusterMembership(domainName, hostName);
            if (! proxy.isRegistered(cmon)) {
                logger.log(BasicLevel.ERROR, "CatalinaClusterMembership not registered:" + hostName);
                return false;
            }
            // These info are about the Cluster itself, not the Member.
            // TODO set the other parameters.
            String mcastAddr = (String) proxy.getAttribute(cmon, "mcastAddr");
            cluster.setMcastAddr(mcastAddr);

            // Register the MBean if not done
            if (!mbeanServer.isRegistered(clon)) {
                try {
                    // A MBean is registered for each Cluster
                    mbeanServer.registerMBean(cluster, clon);
                } catch (Exception e) {
                    logger.log(BasicLevel.ERROR, "Cannot register tomcat cluster:" + e);
                    return false;
                }
            }
            myclusters.put(clusterName, cluster);
        }

        // add a server to the cluster
        return cluster.addTomcatServer(serverName, hostName, proxy);
    }

    public Collection getClusterList() {
        return myclusters.values();
    }

}
